/*
 * Copyright 2018 SMART FX sp. z o.o. All rights reserved.
 * SMARTFX PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package strategies.sample;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.dukascopy.api.IContext;
import com.dukascopy.api.IOrder;
import com.dukascopy.api.JFException;

public class Operations {

	public final long CLOSURE_COMMNAND_REPEAT_GAP = 2500L;

	private IContext context;
	private double equityTP;
	private Map<String, Long> lastClosureTrialTime;

	public Operations(IContext context, double equityTP) {
		
		this.context = context;
		this.equityTP = equityTP;

		lastClosureTrialTime = new HashMap<>();
	}

	public void actionOnTakeProfitLevelReached() throws JFException {

		List<IOrder> allOrders = context.getEngine().getOrders();

		if (allOrders.isEmpty()) {
			return;
		}

		List<IOrder> clousreCandidates = new ArrayList<>();

		for (IOrder order : allOrders) {
			if (order.getState() == IOrder.State.FILLED || order.getState() == IOrder.State.OPENED) {
				clousreCandidates.add(order);
			}
		}

		if (clousreCandidates.isEmpty()) {
			return;
		}

		context.getConsole().getOut().printf("Current equity (%.2f) reached defined equity take profit level (%.2f) /n",
				context.getAccount().getEquity(), equityTP);

		context.getConsole().getOut().println("Closing all orders");

		closeAllOrders(clousreCandidates);
	}

	private void closeAllOrders(List<IOrder> orders) {

		for (IOrder order : orders) {

			String orderID = order.getId();

			if (lastClosureTrialTime.containsKey(orderID)) {
				if ((System.currentTimeMillis() - lastClosureTrialTime.get(orderID)) > CLOSURE_COMMNAND_REPEAT_GAP) {
					closeOrder(order, orderID);
				}
			} else {
				closeOrder(order, orderID);
			}
		}

	}

	private void closeOrder(IOrder order, String orderID) {

		lastClosureTrialTime.put(orderID, System.currentTimeMillis());

		try {
			order.close();
		} catch (Exception e) {
			context.getConsole().getErr().printf("Failure on closure order: %s (%s) /n", order.toString(), e.getMessage());
			context.getConsole().getErr().printf("Will try to close order again in %d millisenconds /n", CLOSURE_COMMNAND_REPEAT_GAP);
		}

	}

}
